#ifndef INPUTD_GUARD_UUID_C65EEF65_C268_4A62_9294_C34FCDDFBE2C
#define INPUTD_GUARD_UUID_C65EEF65_C268_4A62_9294_C34FCDDFBE2C 1

#include <glib.h>
#include <linux/input.h>
#include <linux/uinput.h>

#include "keys.h"

#define INPUTD_DEVICE_PREFIX "[inputd] "
#define INPUTD_PRIORITY -10

#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

#define EV_PRESSED(ev) (ev->value > 0)
#define CFGTR_LAST_KEY(cfgtr) (cfgtr->key[cfgtr->key_count - 1])

enum {
    VERBOSITY_NONE = 0,
    VERBOSITY_INFO = 1,
    VERBOSITY_DEBUG = 2,
};

typedef struct {
    gboolean pressed;
} InputKey;

typedef struct {
    /* Device properties */
    gchar name[256];
    gchar phys[256];
    gchar path[256];
    struct input_id id;

    /* Event source */
    GIOChannel *evsource;
    guint evsource_watch;

    /* uinput */
    gchar uinput_cookie[64];
    GIOChannel *uinput;
    gboolean mouse_emulation;

    /* Current status */
    InputKey key[KEY_MAX];
} InputDevice;

typedef struct {
    KeyDefinition *def;
    gboolean pressed;
} ConfigKey;

typedef struct {
    gint key_count;
    ConfigKey *key;
    ConfigKey target;
    gboolean active;
} ConfigTranslation;

typedef struct {
    gint verbosity;
    gchar *uinput_dev;
    gchar *input_dev_dir;
    GList *translations;
    guint quit_timeout;
} Config;

extern Config config;

#endif

/* vim: set sw=4 sts=4 et foldmethod=marker : */
