/* inputd
 *
 * Copyright (C) 2006, 2007 Michael Hanselmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include "../config.h"

#include <glib.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <errno.h>

#include "devmanager.h"
#include "brain.h"
#include "config.h"
#include "uinput.h"
#include "common.h"

static GMainLoop *main_loop = NULL;

void sigint(int signum)
{
    if (config.verbosity >= VERBOSITY_DEBUG) {
        g_message("Received SIGINT, quitting ...");
    }

    g_main_loop_quit(main_loop);
}

gboolean quit_timeout(gpointer data)
{
    if (config.verbosity >= VERBOSITY_DEBUG) {
        g_message("Quit timeout over, quitting ...");
    }

    g_main_loop_quit(main_loop);

    return FALSE;
}

int main(int argc, char *argv[])
{
    if (!config_parse_cmdline(argc, argv)) {
        return 1;
    }

    /* Renice ourself to get a higher priority */
    if (setpriority(PRIO_PROCESS, 0, INPUTD_PRIORITY) < 0) {
        char *msg = g_strconcat("setpriority(): ", strerror(errno), NULL);
        g_warning(msg);
        g_free(msg);
    }

    main_loop = g_main_loop_new(NULL, FALSE);

    /* Initialize components */
    if (!config_init()) {
        g_error("Failed to initialize config");
    }

    if (!uinput_init()) {
        g_error("Failed to initialize uinput");
    }

    if (!brain_init()) {
        g_error("Failed to initialize brain");
    }

    if (!devmanager_init()) {
        g_error("Failed to initialize device manager");
    }

    /* Initialize timeout */
    if (config.quit_timeout > 0) {
        guint timeout;

        if (config.quit_timeout > (G_MAXUINT / 1000)) {
            timeout = G_MAXUINT;
        } else {
            timeout = config.quit_timeout * 1000;
        }

        g_timeout_add(timeout, quit_timeout, NULL);
    }

    /* Setup signal handlers */
    signal(SIGINT, sigint);

    /* Run main loop */
    g_main_loop_run(main_loop);

    /* Reset signal handlers */
    signal(SIGINT, SIG_DFL);

    /* Exit components */
    devmanager_exit();
    brain_exit();
    uinput_exit();
    config_exit();

    return 0;
}

/* vim: set sw=4 sts=4 et foldmethod=marker : */
