/* inputd
 *
 * Copyright (C) 2006, 2007 Michael Hanselmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include "../config.h"

#include <glib.h>
#include <string.h>

#include "common.h"

KeyDefinition *key_by_code(const guint16 code)
{
    KeyDefinition *kd = &key_defs[0];

    while (kd && kd->name) {
        if (kd->code == code) {
            return kd;
        }

        ++kd;
    }

    return NULL;
}

KeyDefinition *key_by_name(const gchar *name)
{
    KeyDefinition *kd = &key_defs[0];

    while (kd && kd->name) {
        if (strcmp(kd->name, name) == 0) {
            return kd;
        }

        ++kd;
    }

    return NULL;
}

/* In this file, we employ some macro magic to make our lives easier. */

#define DEF_KEY(_key, _flags) \
    { .name = (#_key), .code = (_key), .flags = (_flags) }

KeyDefinition key_defs[] = {
    DEF_KEY(BTN_0, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_1, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_3, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_4, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_5, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_6, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_7, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_8, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_9, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_A, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_B, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BACK, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BASE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BASE2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BASE3, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BASE4, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BASE5, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_BASE6, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_C, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_DEAD, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_DIGI, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_EXTRA, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_FORWARD, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_GAMEPAD, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_GEAR_DOWN, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_GEAR_UP, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_JOYSTICK, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_LEFT, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_MIDDLE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_MISC, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_MODE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_MOUSE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_PINKIE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_RIGHT, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_SELECT, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_SIDE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_START, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_STYLUS, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_STYLUS2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TASK, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_THUMB, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_THUMB2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_THUMBL, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_THUMBR, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TL, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TL2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_AIRBRUSH, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_BRUSH, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_DOUBLETAP, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_FINGER, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_LENS, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_MOUSE, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_PEN, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_PENCIL, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_RUBBER, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOOL_TRIPLETAP, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOP, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOP2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TOUCH, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TR, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TR2, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_TRIGGER, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_WHEEL, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_X, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_Y, KEY_FLAG_BUTTON),
    DEF_KEY(BTN_Z, KEY_FLAG_BUTTON),
    DEF_KEY(KEY_0, 0),
    DEF_KEY(KEY_1, 0),
    DEF_KEY(KEY_102ND, 0),
    DEF_KEY(KEY_2, 0),
    DEF_KEY(KEY_3, 0),
    DEF_KEY(KEY_4, 0),
    DEF_KEY(KEY_5, 0),
    DEF_KEY(KEY_6, 0),
    DEF_KEY(KEY_7, 0),
    DEF_KEY(KEY_8, 0),
    DEF_KEY(KEY_9, 0),
    DEF_KEY(KEY_A, 0),
    DEF_KEY(KEY_AB, 0),
    DEF_KEY(KEY_AGAIN, 0),
    DEF_KEY(KEY_ALTERASE, 0),
    DEF_KEY(KEY_ANGLE, 0),
    DEF_KEY(KEY_APOSTROPHE, 0),
    DEF_KEY(KEY_ARCHIVE, 0),
    DEF_KEY(KEY_AUDIO, 0),
    DEF_KEY(KEY_AUX, 0),
    DEF_KEY(KEY_B, 0),
    DEF_KEY(KEY_BACK, 0),
    DEF_KEY(KEY_BACKSLASH, 0),
    DEF_KEY(KEY_BACKSPACE, 0),
    DEF_KEY(KEY_BASSBOOST, 0),
    DEF_KEY(KEY_BLUE, 0),
    DEF_KEY(KEY_BOOKMARKS, 0),
    DEF_KEY(KEY_BREAK, 0),
    DEF_KEY(KEY_BRIGHTNESSDOWN, 0),
    DEF_KEY(KEY_BRIGHTNESSUP, 0),
    DEF_KEY(KEY_C, 0),
    DEF_KEY(KEY_CALC, 0),
    DEF_KEY(KEY_CALENDAR, 0),
    DEF_KEY(KEY_CAMERA, 0),
    DEF_KEY(KEY_CANCEL, 0),
    DEF_KEY(KEY_CAPSLOCK, 0),
    DEF_KEY(KEY_CD, 0),
    DEF_KEY(KEY_CHANNEL, 0),
    DEF_KEY(KEY_CHANNELDOWN, 0),
    DEF_KEY(KEY_CHANNELUP, 0),
    DEF_KEY(KEY_CHAT, 0),
    DEF_KEY(KEY_CLEAR, 0),
    DEF_KEY(KEY_CLOSE, 0),
    DEF_KEY(KEY_CLOSECD, 0),
    DEF_KEY(KEY_COFFEE, 0),
    DEF_KEY(KEY_COMMA, 0),
    DEF_KEY(KEY_COMPOSE, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_COMPUTER, 0),
    DEF_KEY(KEY_CONFIG, 0),
    DEF_KEY(KEY_CONNECT, 0),
    DEF_KEY(KEY_COPY, 0),
    DEF_KEY(KEY_CUT, 0),
    DEF_KEY(KEY_CYCLEWINDOWS, 0),
    DEF_KEY(KEY_D, 0),
    DEF_KEY(KEY_DEL_EOL, 0),
    DEF_KEY(KEY_DEL_EOS, 0),
    DEF_KEY(KEY_DELETE, 0),
    DEF_KEY(KEY_DELETEFILE, 0),
    DEF_KEY(KEY_DEL_LINE, 0),
    DEF_KEY(KEY_DIGITS, 0),
    DEF_KEY(KEY_DIRECTION, 0),
    DEF_KEY(KEY_DIRECTORY, 0),
    DEF_KEY(KEY_DOCUMENTS, 0),
    DEF_KEY(KEY_DOT, 0),
    DEF_KEY(KEY_DOWN, 0),
    DEF_KEY(KEY_DVD, 0),
    DEF_KEY(KEY_E, 0),
    DEF_KEY(KEY_EDIT, 0),
    DEF_KEY(KEY_EJECTCD, 0),
    DEF_KEY(KEY_EJECTCLOSECD, 0),
    DEF_KEY(KEY_EMAIL, 0),
    DEF_KEY(KEY_END, 0),
    DEF_KEY(KEY_ENTER, 0),
    DEF_KEY(KEY_EPG, 0),
    DEF_KEY(KEY_EQUAL, 0),
    DEF_KEY(KEY_ESC, 0),
    DEF_KEY(KEY_EXIT, 0),
    DEF_KEY(KEY_F, 0),
    DEF_KEY(KEY_F1, 0),
    DEF_KEY(KEY_F10, 0),
    DEF_KEY(KEY_F11, 0),
    DEF_KEY(KEY_F12, 0),
    DEF_KEY(KEY_F13, 0),
    DEF_KEY(KEY_F14, 0),
    DEF_KEY(KEY_F15, 0),
    DEF_KEY(KEY_F16, 0),
    DEF_KEY(KEY_F17, 0),
    DEF_KEY(KEY_F18, 0),
    DEF_KEY(KEY_F19, 0),
    DEF_KEY(KEY_F2, 0),
    DEF_KEY(KEY_F20, 0),
    DEF_KEY(KEY_F21, 0),
    DEF_KEY(KEY_F22, 0),
    DEF_KEY(KEY_F23, 0),
    DEF_KEY(KEY_F24, 0),
    DEF_KEY(KEY_F3, 0),
    DEF_KEY(KEY_F4, 0),
    DEF_KEY(KEY_F5, 0),
    DEF_KEY(KEY_F6, 0),
    DEF_KEY(KEY_F7, 0),
    DEF_KEY(KEY_F8, 0),
    DEF_KEY(KEY_F9, 0),
    DEF_KEY(KEY_FASTFORWARD, 0),
    DEF_KEY(KEY_FAVORITES, 0),
    DEF_KEY(KEY_FILE, 0),
    DEF_KEY(KEY_FINANCE, 0),
    DEF_KEY(KEY_FIND, 0),
    DEF_KEY(KEY_FIRST, 0),
    DEF_KEY(KEY_FN, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_FN_1, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_FN_2, KEY_FLAG_MODIFIER),
    /* Not sure whether the following KEY_FN_* keys are modifiers, too */
    DEF_KEY(KEY_FN_B, 0),
    DEF_KEY(KEY_FN_D, 0),
    DEF_KEY(KEY_FN_E, 0),
    DEF_KEY(KEY_FN_ESC, 0),
    DEF_KEY(KEY_FN_F, 0),
    DEF_KEY(KEY_FN_F1, 0),
    DEF_KEY(KEY_FN_F10, 0),
    DEF_KEY(KEY_FN_F11, 0),
    DEF_KEY(KEY_FN_F12, 0),
    DEF_KEY(KEY_FN_F2, 0),
    DEF_KEY(KEY_FN_F3, 0),
    DEF_KEY(KEY_FN_F4, 0),
    DEF_KEY(KEY_FN_F5, 0),
    DEF_KEY(KEY_FN_F6, 0),
    DEF_KEY(KEY_FN_F7, 0),
    DEF_KEY(KEY_FN_F8, 0),
    DEF_KEY(KEY_FN_F9, 0),
    DEF_KEY(KEY_FN_S, 0),
    DEF_KEY(KEY_FORWARD, 0),
    DEF_KEY(KEY_FORWARDMAIL, 0),
    DEF_KEY(KEY_FRONT, 0),
    DEF_KEY(KEY_G, 0),
    DEF_KEY(KEY_GOTO, 0),
    DEF_KEY(KEY_GRAVE, 0),
    DEF_KEY(KEY_GREEN, 0),
    DEF_KEY(KEY_H, 0),
    DEF_KEY(KEY_HANGUEL, 0),
    DEF_KEY(KEY_HANJA, 0),
    DEF_KEY(KEY_HELP, 0),
    DEF_KEY(KEY_HENKAN, 0),
    DEF_KEY(KEY_HIRAGANA, 0),
    DEF_KEY(KEY_HOME, 0),
    DEF_KEY(KEY_HOMEPAGE, 0),
    DEF_KEY(KEY_HP, 0),
    DEF_KEY(KEY_I, 0),
    DEF_KEY(KEY_INFO, 0),
    DEF_KEY(KEY_INSERT, 0),
    DEF_KEY(KEY_INS_LINE, 0),
    DEF_KEY(KEY_ISO, 0),
    DEF_KEY(KEY_J, 0),
    DEF_KEY(KEY_K, 0),
    DEF_KEY(KEY_KATAKANA, 0),
    DEF_KEY(KEY_KATAKANAHIRAGANA, 0),
    DEF_KEY(KEY_KBDILLUMDOWN, 0),
    DEF_KEY(KEY_KBDILLUMTOGGLE, 0),
    DEF_KEY(KEY_KBDILLUMUP, 0),
    DEF_KEY(KEY_KEYBOARD, 0),
    DEF_KEY(KEY_KP0, 0),
    DEF_KEY(KEY_KP1, 0),
    DEF_KEY(KEY_KP2, 0),
    DEF_KEY(KEY_KP3, 0),
    DEF_KEY(KEY_KP4, 0),
    DEF_KEY(KEY_KP5, 0),
    DEF_KEY(KEY_KP6, 0),
    DEF_KEY(KEY_KP7, 0),
    DEF_KEY(KEY_KP8, 0),
    DEF_KEY(KEY_KP9, 0),
    DEF_KEY(KEY_KPASTERISK, 0),
    DEF_KEY(KEY_KPCOMMA, 0),
    DEF_KEY(KEY_KPDOT, 0),
    DEF_KEY(KEY_KPENTER, 0),
    DEF_KEY(KEY_KPEQUAL, 0),
    DEF_KEY(KEY_KPJPCOMMA, 0),
    DEF_KEY(KEY_KPLEFTPAREN, 0),
    DEF_KEY(KEY_KPMINUS, 0),
    DEF_KEY(KEY_KPPLUS, 0),
    DEF_KEY(KEY_KPPLUSMINUS, 0),
    DEF_KEY(KEY_KPRIGHTPAREN, 0),
    DEF_KEY(KEY_KPSLASH, 0),
    DEF_KEY(KEY_L, 0),
    DEF_KEY(KEY_LANGUAGE, 0),
    DEF_KEY(KEY_LAST, 0),
    DEF_KEY(KEY_LEFT, 0),
    DEF_KEY(KEY_LEFTALT, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_LEFTBRACE, 0),
    DEF_KEY(KEY_LEFTCTRL, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_LEFTMETA, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_LEFTSHIFT, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_LINEFEED, 0),
    DEF_KEY(KEY_LIST, 0),
    DEF_KEY(KEY_M, 0),
    DEF_KEY(KEY_MACRO, 0),
    DEF_KEY(KEY_MAIL, 0),
    DEF_KEY(KEY_MEDIA, 0),
    DEF_KEY(KEY_MEMO, 0),
    DEF_KEY(KEY_MENU, 0),
    DEF_KEY(KEY_MHP, 0),
    DEF_KEY(KEY_MINUS, 0),
    DEF_KEY(KEY_MODE, 0),
    DEF_KEY(KEY_MOVE, 0),
    DEF_KEY(KEY_MP3, 0),
    DEF_KEY(KEY_MSDOS, 0),
    DEF_KEY(KEY_MUHENKAN, 0),
    DEF_KEY(KEY_MUTE, 0),
    DEF_KEY(KEY_N, 0),
    DEF_KEY(KEY_NEW, 0),
    DEF_KEY(KEY_NEXT, 0),
    DEF_KEY(KEY_NEXTSONG, 0),
    DEF_KEY(KEY_NUMLOCK, 0),
    DEF_KEY(KEY_O, 0),
    DEF_KEY(KEY_OK, 0),
    DEF_KEY(KEY_OPEN, 0),
    DEF_KEY(KEY_OPTION, 0),
    DEF_KEY(KEY_P, 0),
    DEF_KEY(KEY_PAGEDOWN, 0),
    DEF_KEY(KEY_PAGEUP, 0),
    DEF_KEY(KEY_PASTE, 0),
    DEF_KEY(KEY_PAUSE, 0),
    DEF_KEY(KEY_PAUSECD, 0),
    DEF_KEY(KEY_PC, 0),
    DEF_KEY(KEY_PHONE, 0),
    DEF_KEY(KEY_PLAY, 0),
    DEF_KEY(KEY_PLAYCD, 0),
    DEF_KEY(KEY_PLAYER, 0),
    DEF_KEY(KEY_PLAYPAUSE, 0),
    DEF_KEY(KEY_POWER, 0),
    DEF_KEY(KEY_POWER2, 0),
    DEF_KEY(KEY_PREVIOUS, 0),
    DEF_KEY(KEY_PREVIOUSSONG, 0),
    DEF_KEY(KEY_PRINT, 0),
    DEF_KEY(KEY_PROG1, 0),
    DEF_KEY(KEY_PROG2, 0),
    DEF_KEY(KEY_PROG3, 0),
    DEF_KEY(KEY_PROG4, 0),
    DEF_KEY(KEY_PROGRAM, 0),
    DEF_KEY(KEY_PROPS, 0),
    DEF_KEY(KEY_PVR, 0),
    DEF_KEY(KEY_Q, 0),
    DEF_KEY(KEY_QUESTION, 0),
    DEF_KEY(KEY_R, 0),
    DEF_KEY(KEY_RADIO, 0),
    DEF_KEY(KEY_RECORD, 0),
    DEF_KEY(KEY_RED, 0),
    DEF_KEY(KEY_REDO, 0),
    DEF_KEY(KEY_REFRESH, 0),
    DEF_KEY(KEY_REPLY, 0),
    DEF_KEY(KEY_RESTART, 0),
    DEF_KEY(KEY_REWIND, 0),
    DEF_KEY(KEY_RIGHT, 0),
    DEF_KEY(KEY_RIGHTALT, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_RIGHTBRACE, 0),
    DEF_KEY(KEY_RIGHTCTRL, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_RIGHTMETA, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_RIGHTSHIFT, KEY_FLAG_MODIFIER),
    DEF_KEY(KEY_RO, 0),
    DEF_KEY(KEY_S, 0),
    DEF_KEY(KEY_SAT, 0),
    DEF_KEY(KEY_SAT2, 0),
    DEF_KEY(KEY_SAVE, 0),
    DEF_KEY(KEY_SCREEN, 0),
    DEF_KEY(KEY_SCROLLDOWN, 0),
    DEF_KEY(KEY_SCROLLLOCK, 0),
    DEF_KEY(KEY_SCROLLUP, 0),
    DEF_KEY(KEY_SEARCH, 0),
    DEF_KEY(KEY_SELECT, 0),
    DEF_KEY(KEY_SEMICOLON, 0),
    DEF_KEY(KEY_SEND, 0),
    DEF_KEY(KEY_SENDFILE, 0),
    DEF_KEY(KEY_SETUP, 0),
    DEF_KEY(KEY_SHOP, 0),
    DEF_KEY(KEY_SHUFFLE, 0),
    DEF_KEY(KEY_SLASH, 0),
    DEF_KEY(KEY_SLEEP, 0),
    DEF_KEY(KEY_SLOW, 0),
    DEF_KEY(KEY_SOUND, 0),
    DEF_KEY(KEY_SPACE, 0),
    DEF_KEY(KEY_SPORT, 0),
    DEF_KEY(KEY_STOP, 0),
    DEF_KEY(KEY_STOPCD, 0),
    DEF_KEY(KEY_SUBTITLE, 0),
    DEF_KEY(KEY_SUSPEND, 0),
    DEF_KEY(KEY_SWITCHVIDEOMODE, 0),
    DEF_KEY(KEY_SYSRQ, 0),
    DEF_KEY(KEY_T, 0),
    DEF_KEY(KEY_TAB, 0),
    DEF_KEY(KEY_TAPE, 0),
    DEF_KEY(KEY_TEEN, 0),
    DEF_KEY(KEY_TEXT, 0),
    DEF_KEY(KEY_TIME, 0),
    DEF_KEY(KEY_TITLE, 0),
    DEF_KEY(KEY_TUNER, 0),
    DEF_KEY(KEY_TV, 0),
    DEF_KEY(KEY_TV2, 0),
    DEF_KEY(KEY_TWEN, 0),
    DEF_KEY(KEY_U, 0),
    DEF_KEY(KEY_UNDO, 0),
    DEF_KEY(KEY_UNKNOWN, 0),
    DEF_KEY(KEY_UP, 0),
    DEF_KEY(KEY_V, 0),
    DEF_KEY(KEY_VCR, 0),
    DEF_KEY(KEY_VCR2, 0),
    DEF_KEY(KEY_VENDOR, 0),
    DEF_KEY(KEY_VIDEO, 0),
    DEF_KEY(KEY_VOLUMEDOWN, 0),
    DEF_KEY(KEY_VOLUMEUP, 0),
    DEF_KEY(KEY_W, 0),
    DEF_KEY(KEY_WAKEUP, 0),
    DEF_KEY(KEY_WWW, 0),
    DEF_KEY(KEY_X, 0),
    DEF_KEY(KEY_XFER, 0),
    DEF_KEY(KEY_Y, 0),
    DEF_KEY(KEY_YELLOW, 0),
    DEF_KEY(KEY_YEN, 0),
    DEF_KEY(KEY_Z, 0),
    DEF_KEY(KEY_ZENKAKUHANKAKU, 0),
    DEF_KEY(KEY_ZOOM, 0),
    { NULL }
};

/* vim: set sw=4 sts=4 et foldmethod=marker : */
