/* inputd
 *
 * Copyright (C) 2006, 2007 Michael Hanselmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <glib.h>
#include <glib/gprintf.h>

#include <time.h>

#include "utils.h"

/* {{{ Format date/time value */
gchar *format_datetime(const GTimeVal *tv)
{
    struct tm *tm = localtime(&tv->tv_sec);

    return g_strdup_printf("%04d-%02d-%02d %02d:%02d:%02d.%04ld [",
                           1900 + tm->tm_year, 1 + tm->tm_mon, tm->tm_mday,
                           tm->tm_hour, tm->tm_min, tm->tm_sec,
                           (glong)(tv->tv_usec * 10000 / G_USEC_PER_SEC));
}
/* }}} */

/* {{{ Event dumper */
void dump_event(const struct input_event *ev)
{
    GTimeVal now;
    gchar *temp;

    switch (ev->type) {
        case EV_KEY:
            break;

        default:
            return;
    }

    g_get_current_time(&now);
    temp = format_datetime(&now);
    g_print(temp);
    g_free(temp);

    switch (ev->type) {
        case EV_SYN:
            g_print("EV_SYN");
            break;

        case EV_KEY:
            g_print("EV_KEY");
            break;

        case EV_REL:
            g_print("EV_REL");
            break;

        case EV_ABS:
            g_print("EV_ABS");
            break;

        default:
            g_printf("%d", ev->type);
            break;
    }

    g_print(", ");

    switch (ev->type) {
        case EV_KEY: {
            KeyDefinition *kd = key_by_code(ev->code);
            if (kd == NULL) {
                g_print("UNK");
            } else {
                g_print(kd->name);
            }
            break;
        }

        default:
            g_printf("%d", ev->code);
            break;
    }

    g_printf(", %d]\n", ev->value);
}
/* }}} */

/* {{{ Translation dumper */
void dump_translation(const ConfigTranslation *cfgtr)
{
    gchar *keys;

#define CFGTR_KEY_FORMAT "%3d"
#define CFGTR_KEY(idx) cfgtr->key[idx].def->code

    switch (cfgtr->key_count) {
    case 0:
        keys = g_strdup("");
        break;
    case 1:
        keys = g_strdup_printf(CFGTR_KEY_FORMAT,
            CFGTR_KEY(0));
        break;
    case 2:
        keys = g_strdup_printf(CFGTR_KEY_FORMAT " + "
                               CFGTR_KEY_FORMAT,
                               CFGTR_KEY(0),
                               CFGTR_KEY(1));
        break;
    case 3:
        keys = g_strdup_printf(CFGTR_KEY_FORMAT " + "
                               CFGTR_KEY_FORMAT " + "
                               CFGTR_KEY_FORMAT,
                               CFGTR_KEY(0),
                               CFGTR_KEY(1),
                               CFGTR_KEY(2));
        break;
    case 4:
        keys = g_strdup_printf(CFGTR_KEY_FORMAT " + "
                               CFGTR_KEY_FORMAT " + "
                               CFGTR_KEY_FORMAT " + "
                               CFGTR_KEY_FORMAT,
                               CFGTR_KEY(0),
                               CFGTR_KEY(1),
                               CFGTR_KEY(2),
                               CFGTR_KEY(3));
        break;
    default:
        keys = g_strdup_printf("TODO");
        break;
    }

#undef CFGTR_KEY_FORMAT
#undef CFGTR_KEY

    g_printf("%s = %d (active = %d)\n",
             keys,
             cfgtr->target.def->code,
             cfgtr->active);

    g_free(keys);
}
/* }}} */

/* {{{ Configuration dumper */
static void dump_config_translation(gpointer data, gpointer user_data)
{
    dump_translation(data);
}

void dump_config()
{
    g_print("Current configuration:\n");
    g_printf("  verbosity = %d\n", config.verbosity);
    g_printf("  uinput_dev = %s\n", config.uinput_dev);
    g_printf("  input_dev_dir = %s\n", config.input_dev_dir);
    g_printf("  quit_timeout = %d\n", config.quit_timeout);

    g_printf("Translations:\n");
    g_list_foreach(config.translations, dump_config_translation, NULL);
}
/* }}} */

/* vim: set sw=4 sts=4 et foldmethod=marker : */
